@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
 
 import argparse
-import functools
 import glob
 import locale
 import os
@@ -13,7 +12,7 @@ import uuid
 import parso
 import tbtrim
 
-__all__ = ['poseur', 'convert', 'decorator']
+__all__ = ['poseur', 'convert', 'decorator']  # pylint: disable=undefined-all-variable
 
 # multiprocessing may not be supported
 try:        # try first
@@ -32,7 +31,7 @@ finally:    # alias and aftermath
     del multiprocessing
 
 # version string
-__version__ = '0.2.1'
+__version__ = '0.3.4'
 
 # from configparser
 BOOLEAN_STATES = {'1': True, '0': False,
@@ -73,9 +72,8 @@ tbtrim.set_trim_rule(predicate, strict=True, target=ConvertError)
 ###############################################################################
 # Positional-only decorator
 
-
 _decorator = '''
-def _poseur_decorator(*poseur):
+def %s(*poseur):
     """Positional-only arguments runtime checker.
 
     Args:
@@ -91,35 +89,14 @@ def _poseur_decorator(*poseur):
         def wrapper(*args, **kwargs):
             poseur_args = set(poseur).intersection(kwargs)
             if poseur_args:
-                raise TypeError('%s() got some positional-only arguments passed as keyword arguments: %r' %
-                                (func.__name__, ', '.join(poseur_args)))
-            return func(*args, **kwargs)
-        return wrapper
-    return caller
-'''
-
-
-def decorator(*poseurs):
-    """Positional-only arguments runtime checker.
-
-    Args:
-     - `*poseurs` -- `str`, name of positional-only arguments
-
-    Refs:
-     - https://mail.python.org/pipermail/python-ideas/2017-February/044888.html
-
-    """
-    def caller(func):
-        @functools.wraps(func)
-        def wrapper(*args, **kwargs):
-            poseur_args = set(poseurs).intersection(kwargs)
-            if poseur_args:
-                raise TypeError('%s() got some positional-only arguments passed as keyword arguments: %r' %
+                raise TypeError('%%s() got some positional-only arguments passed as keyword arguments: %%r' %%
                                 (func.__name__, ', '.join(poseur_args)))
             return func(*args, **kwargs)
         return wrapper
     return caller
+'''.splitlines()
 
+exec(os.linesep.join(_decorator) % 'decorator')
 
 ###############################################################################
 # Main convertion implementation
@@ -159,20 +136,26 @@ def decorate_lambdef(parameters, lambdef):
      - `parameters` -- `List[parso.python.tree.Param]`, extracted positional-only arguments
      - `lambdef` -- `str`, converted lambda string
 
+    Envs:
+     - `POSEUR_DECORATOR` -- name of decorator for runtime checks (same as `--decorator` option in CLI)
+
     Returns:
      - `str` -- decorated lambda definition
 
     """
+    POSEUR_DECORATOR = os.getenv('POSEUR_DECORATOR', __poseur_decorator__)
+
     match_prefix = re.match(r'^(?P<prefix>\s*).*?', lambdef, re.DOTALL | re.MULTILINE)
     prefix = '' if match_prefix is None else match_prefix.group('prefix')
 
     match_suffix = re.match(r'.*?(?P<suffix>\s*)$', lambdef, re.DOTALL | re.MULTILINE)
     suffix = '' if match_suffix is None else match_suffix.group('suffix')
 
-    return '%s_poseur_decorator(%s)(%s)%s' % (prefix,
-                                              ', '.join(map(lambda param: repr(param.name.value), parameters)),
-                                              lambdef.strip(),
-                                              suffix)
+    return '%s%s(%s)(%s)%s' % (prefix,
+                               POSEUR_DECORATOR,
+                               ', '.join(map(lambda param: repr(param.name.value), parameters)),
+                               lambdef.strip(),
+                               suffix)
 
 
 def dismiss_lambdef(node):
@@ -261,6 +244,9 @@ def process_lambdef(node, flag):
      - `node` -- `parso.python.tree.Lambda`, lambda AST
      - `flag` -- `bool`, dismiss runtime checks for positional-only arguments (same as `--dismiss` option in CLI)
 
+    Envs:
+     - `POSEUR_DECORATOR` -- name of decorator for runtime checks (same as `--decorator` option in CLI)
+
     Returns:
      - `str` -- processed source string
 
@@ -288,31 +274,34 @@ def decorate_funcdef(parameters, column, funcdef):
      - `funcdef` -- `str`, converted function string
 
     Envs:
-     - `POSEUR_LINSEP` -- line separator to process source files (same as `--linesep` option in CLI)
+     - `POSEUR_LINESEP` -- line separator to process source files (same as `--linesep` option in CLI)
+     - `POSEUR_DECORATOR` -- name of decorator for runtime checks (same as `--decorator` option in CLI)
 
     Returns:
      - `str` -- decorated function definition
 
     """
-    POSEUR_LINESEP = os.getenv('POSEUR_LINSEP', os.linesep)
+    POSEUR_LINESEP = os.getenv('POSEUR_LINESEP', os.linesep)
+    POSEUR_DECORATOR = os.getenv('POSEUR_DECORATOR', __poseur_decorator__)
 
     prefix = ''
     suffix = ''
     deflag = False
 
     for line in funcdef.splitlines(True):
-        if re.match(r'^\s*(async)\s+?def.*', line) is not None:
+        if re.match(r'^\s*(async\s+)?def\s', line) is not None:
             deflag = True
         if deflag:
             suffix += line
         else:
             prefix += line
 
-    return '%s%s@_poseur_decorator(%s)%s%s' % (prefix,
-                                               '\t'.expandtabs(column),
-                                               ', '.join(map(lambda param: repr(param.name.value), parameters)),
-                                               POSEUR_LINESEP,
-                                               suffix)
+    return '%s%s@%s(%s)%s%s' % (prefix,
+                                '\t'.expandtabs(column),
+                                POSEUR_DECORATOR,
+                                ', '.join(map(lambda param: repr(param.name.value), parameters)),
+                                POSEUR_LINESEP,
+                                suffix)
 
 
 def dismiss_funcdef(node):
@@ -390,6 +379,9 @@ def process_funcdef(node, flag, *, async_ctx=None):
     Kwds:
      - `async_ctx` -- `parso.python.tree.Keyword`, `async` keyword AST node
 
+    Envs:
+     - `POSEUR_DECORATOR` -- name of decorator for runtime checks (same as `--decorator` option in CLI)
+
     Returns:
      - `str` -- processed source string
 
@@ -509,6 +501,33 @@ def find_poseur(node, root=0):
     return -1
 
 
+def check_suffix(string):
+    """Strip comments from string.
+
+    Args:
+     - `string` -- `str`, buffer string
+
+    Returns:
+     - `str` -- prefix comments
+     - `str` -- suffix strings
+
+    """
+    prefix = ''
+    suffix = ''
+
+    lines = iter(string.splitlines(True))
+    for line in lines:
+        if line.strip().startswith('#'):
+            prefix += line
+            continue
+        suffix += line
+        break
+
+    for line in lines:
+        suffix += line
+    return prefix, suffix
+
+
 def process_module(node):
     """Walk top nodes of the AST module.
 
@@ -516,7 +535,9 @@ def process_module(node):
      - `node` -- `parso.python.tree.Module`, parso AST
 
     Envs:
+     - `POSEUR_LINESEP` -- line separator to process source files (same as `--linesep` option in CLI)
      - `POSEUR_LINTING` -- lint converted codes (same as `--linting` option in CLI)
+     - `POSEUR_DECORATOR` -- name of decorator for runtime checks (same as `--decorator` option in CLI)
 
     Returns:
      - `str` -- processed source string
@@ -530,10 +551,15 @@ def process_module(node):
         if index < postmt:
             prefix += walk(child)
         else:
-            suffix += walk(child)
+            bufpre, bufsuf = check_suffix(walk(child))
+            prefix += bufpre
+            suffix += bufsuf
 
     if postmt >= 0:
-        middle = _decorator
+        POSEUR_LINESEP = os.getenv('POSEUR_LINESEP', os.linesep)
+        POSEUR_DECORATOR = os.getenv('POSEUR_DECORATOR', __poseur_decorator__)
+
+        middle = POSEUR_LINESEP.join(_decorator) % POSEUR_DECORATOR
         if not prefix:
             middle = middle.lstrip()
         if not suffix:
@@ -541,7 +567,6 @@ def process_module(node):
 
         POSEUR_LINTING = BOOLEAN_STATES.get(os.getenv('POSEUR_LINTING', '0').casefold(), False)
         if POSEUR_LINTING:
-            POSEUR_LINESEP = os.getenv('POSEUR_LINSEP', os.linesep)
             if prefix:
                 if prefix.endswith(POSEUR_LINESEP * 2):
                     pass
@@ -581,9 +606,10 @@ def walk(node):
                  parso AST
 
     Envs:
-     - `POSEUR_LINSEP` -- line separator to process source files (same as `--linesep` option in CLI)
+     - `POSEUR_LINESEP` -- line separator to process source files (same as `--linesep` option in CLI)
      - `POSEUR_DISMISS` -- dismiss runtime checks for positional-only arguments (same as `--dismiss` option in CLI)
      - `POSEUR_LINTING` -- lint converted codes (same as `--linting` option in CLI)
+     - `POSEUR_DECORATOR` -- name of decorator for runtime checks (same as `--decorator` option in CLI)
 
     Returns:
      - `str` -- converted string
@@ -631,9 +657,10 @@ def convert(string, source='<unknown>'):
 
     Envs:
      - `POSEUR_VERSION` -- convert against Python version (same as `--python` option in CLI)
-     - `POSEUR_LINSEP` -- line separator to process source files (same as `--linesep` option in CLI)
+     - `POSEUR_LINESEP` -- line separator to process source files (same as `--linesep` option in CLI)
      - `POSEUR_DISMISS` -- dismiss runtime checks for positional-only arguments (same as `--dismiss` option in CLI)
      - `POSEUR_LINTING` -- lint converted codes (same as `--linting` option in CLI)
+     - `POSEUR_DECORATOR` -- name of decorator for runtime checks (same as `--decorator` option in CLI)
 
     Returns:
      - `str` -- converted string
@@ -659,9 +686,10 @@ def poseur(filename):
      - `POSEUR_QUIET` -- run in quiet mode (same as `--quiet` option in CLI)
      - `POSEUR_ENCODING` -- encoding to open source files (same as `--encoding` option in CLI)
      - `POSEUR_VERSION` -- convert against Python version (same as `--python` option in CLI)
-     - `POSEUR_LINSEP` -- line separator to process source files (same as `--linesep` option in CLI)
+     - `POSEUR_LINESEP` -- line separator to process source files (same as `--linesep` option in CLI)
      - `POSEUR_DISMISS` -- dismiss runtime checks for positional-only arguments (same as `--dismiss` option in CLI)
      - `POSEUR_LINTING` -- lint converted codes (same as `--linting` option in CLI)
+     - `POSEUR_DECORATOR` -- name of decorator for runtime checks (same as `--decorator` option in CLI)
 
     """
     POSEUR_QUIET = BOOLEAN_STATES.get(os.getenv('POSEUR_QUIET', '0').casefold(), False)
@@ -692,6 +720,7 @@ __archive__ = os.path.join(__cwd__, 'archive')
 __poseur_version__ = os.getenv('POSEUR_VERSION', POSEUR_VERSION[-1])
 __poseur_encoding__ = os.getenv('POSEUR_ENCODING', LOCALE_ENCODING)
 __poseur_linesep__ = os.getenv('POSEUR_LINESEP', os.linesep)
+__poseur_decorator__ = os.getenv('POSEUR_DECORATOR', '__poseur_decorator')
 
 
 def get_parser():
@@ -728,6 +757,8 @@ def get_parser():
                                help='dismiss runtime checks for positional-only parameters')
     convert_group.add_argument('-nl', '--no-linting', action='store_false', dest='linting',
                                help='do not lint converted codes')
+    convert_group.add_argument('-r', '--decorator', action='store', default=__poseur_decorator__, metavar='VAR',
+                               help='name of decorator for runtime checks (`%s`)' % __poseur_decorator__)
 
     parser.add_argument('file', nargs='+', metavar='SOURCE', default=__cwd__,
                         help='python source files and folders to be converted (%s)' % __cwd__)
@@ -782,9 +813,10 @@ def main(argv=None):
      - `POSEUR_QUIET` -- run in quiet mode (same as `--quiet` option in CLI)
      - `POSEUR_ENCODING` -- encoding to open source files (same as `--encoding` option in CLI)
      - `POSEUR_VERSION` -- convert against Python version (same as `--python` option in CLI)
-     - `POSEUR_LINSEP` -- line separator to process source files (same as `--linesep` option in CLI)
+     - `POSEUR_LINESEP` -- line separator to process source files (same as `--linesep` option in CLI)
      - `POSEUR_DISMISS` -- dismiss runtime checks for positional-only arguments (same as `--dismiss` option in CLI)
      - `POSEUR_LINTING` -- lint converted codes (same as `--linting` option in CLI)
+     - `POSEUR_DECORATOR` -- name of decorator for runtime checks (same as `--decorator` option in CLI)
 
     """
     parser = get_parser()
@@ -794,7 +826,8 @@ def main(argv=None):
     ARCHIVE = args.archive_path
     os.environ['POSEUR_VERSION'] = args.python
     os.environ['POSEUR_ENCODING'] = args.encoding
-    os.environ['POSEUR_LINSEP'] = args.linesep
+    os.environ['POSEUR_LINESEP'] = args.linesep
+    os.environ['POSEUR_DECORATOR'] = args.decorator
     POSEUR_QUIET = os.getenv('POSEUR_QUIET')
     os.environ['POSEUR_QUIET'] = '1' if args.quiet else ('0' if POSEUR_QUIET is None else POSEUR_QUIET)
     POSEUR_DISMISS = os.getenv('POSEUR_DISMISS')
@@ -832,7 +865,8 @@ def main(argv=None):
     if mp is None or CPU_CNT <= 1:
         [poseur(filename) for filename in filelist]  # pylint: disable=expression-not-assigned # pragma: no cover
     else:
-        mp.Pool(processes=CPU_CNT).map(poseur, filelist)
+        with mp.Pool(processes=CPU_CNT) as pool:
+            pool.map(poseur, filelist)
 
 
 if __name__ == '__main__':
